<?php
declare(strict_types = 1);

namespace Drupal\image_processor_webp\EventSubscriber;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Routing\AdminContext;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpKernel\Event\ResponseEvent;
use Symfony\Component\HttpKernel\KernelEvents;

class ImageModifyResponseSubscriber implements EventSubscriberInterface {

    private AdminContext $adminContext;
    private ConfigFactoryInterface $configFactory;


    public function __construct(AdminContext $admin_context, ConfigFactoryInterface $config_factory) {
        $this->adminContext = $admin_context;
        $this->configFactory = $config_factory;
    }


    public static function getSubscribedEvents() {
        return [
            KernelEvents::RESPONSE => ['onResponse', 0],
        ];
    }


    public function onResponse(ResponseEvent $event) {
        $config = $this->configFactory->get('image_processor_webp.settings');
        if (!$config->get('preprocess_enabled')) {
            return;
        }

        if ($this->adminContext->isAdminRoute()) {
            return;
        }

        $request = $event->getRequest();
        $currentPath = $request->getPathInfo();
        if (strpos($currentPath, 'views/ajax') !== false) {
            return;
        }

        $response = $event->getResponse();
        $html = $response->getContent();

        if (!$html) {
            return;
        }

        $doc = new \DOMDocument();
        @$doc->loadHTML($html, LIBXML_HTML_NOIMPLIED | LIBXML_HTML_NODEFDTD);

        $images = $doc->getElementsByTagName('img');

        foreach ($images as $img) {
            $lazySrc = $img->getAttribute('data-src') ?: $img->getAttribute('data-lazyload');
            $src = $lazySrc ?: $img->getAttribute('src');

            // Přeskočení, pokud je zdrojový soubor ve formátu SVG
            if (strtolower(pathinfo($src, PATHINFO_EXTENSION)) === 'svg' || $this->isExternalUrl($src)) {
                continue;
            }

            $format = $config->get('format');
            $path_parts = pathinfo($src);
            if (!array_key_exists('dirname', $path_parts)) {
                \Drupal::logger('image_processor_webp')->error('Missing dirname key: @key', ['@key' => json_encode($path_parts)]);

                continue;
            }

            $webp_path = $path_parts['dirname'] . '/' . $path_parts['filename'] . '.' . $format;

            $fullPath = $_SERVER['DOCUMENT_ROOT'] . $webp_path;
            $fullPath = urldecode($fullPath);

            if (file_exists($fullPath)) {
                $picture = $doc->createElement('picture');
                $source = $doc->createElement('source');
                $source->setAttribute('type', 'image/webp');

                if ($img->hasAttribute('class') && str_contains($img->getAttribute('class'), 'swiper-lazy')) {
                    $source->setAttribute('class', 'swiper-lazy');
                    $source->setAttribute('data-srcset', $webp_path);
                } else {
                    $source->setAttribute('srcset', $webp_path);
                }

                $img_clone = $img->cloneNode();
                $picture->appendChild($source);
                $picture->appendChild($img_clone);
                $img->parentNode->replaceChild($picture, $img);
            } else {
                \Drupal::logger('image_processor_webp')->warning('Missing WEBP: @key', ['@key' => $fullPath]);
            }
        }

        $html = $doc->saveHTML();
        $response->setContent($html);
    }


    private function isExternalUrl($url): bool {
        return (bool)preg_match('/^https?:\/\//', $url);
    }
}
