<?php
declare(strict_types = 1);

namespace Drupal\image_processor_webp;

use Drupal\Core\Config\ImmutableConfig;
use Drupal\file\Entity\File;
use Drupal\image_processor_webp\Form\ImageProcessorSettingsForm;

class ImageProcessor {
    private ImmutableConfig $config;


    public function __construct() {
        $this->config = \Drupal::config('image_processor_webp.settings');
    }


    public function convertToWebp(File $file) {
        if (!$this->config->get('preprocess_enabled')) {
            return;
        }

        $library = $this->config->get('preprocess_library') ?: ImageProcessorSettingsForm::LIBRARY_GD;

        if ($library === ImageProcessorSettingsForm::LIBRARY_GD) {
            $this->convertGdImageWebp($file);
        } elseif ($library === ImageProcessorSettingsForm::LIBRARY_IMAGE_MAGICK) {
            $this->convertImageMagicImageWebp($file);
        }
    }


    private function convertGdImageWebp(File $file) {
        if (!function_exists('imagewebp')) {
            \Drupal::logger('image_processor_webp')->error('GD library not supported WEBP format');

            return;
        }

        $image = $this->createImageResource($file->getFileUri(), $file->getMimeType());
        if (!$image) {
            \Drupal::logger('image_processor_webp')->error('Failed to create image resource from file: @uri', ['@uri' => $file->getFileUri()]);

            return;
        }

        $quality = $this->config->get('webp_quality') ?: 80; // Default to 80 if not set
        $path_parts = pathinfo($file->getFileUri());
        $webp_path = $path_parts['dirname'] . '/' . $path_parts['filename'] . '.webp';

        try {
            if (!imagewebp($image, $webp_path, (int)$quality)) {
                \Drupal::logger('image_processor_webp')->error('Failed to convert to WebP: @uri', ['@uri' => $file->getFileUri()]);
            }
        } catch (\Exception $e) {
            \Drupal::logger('image_processor_webp')->error('Failed to convert to WebP: @uri', ['@uri' => $file->getFileUri()]);
        }

        imagedestroy($image);
    }


    private function convertImageMagicImageWebp(File $file) {
        $imagick = new \Imagick($file->getFileUri());

        try {
            $imagick->setImageFormat('webp');
            $quality = $this->config->get('webp_quality') ?: 80; // Default to 80 if not set
            $imagick->setImageCompressionQuality((int)$quality);

            $path_parts = pathinfo($file->getFileUri());
            $webp_path = $path_parts['dirname'] . '/' . $path_parts['filename'] . '.webp';
            $realpath = \Drupal::service('file_system')->realpath($webp_path);

            if (!$imagick->writeImage($realpath)) {
                \Drupal::logger('image_processor_webp')->error('Failed to convert to WebP using ImageMagick: @uri', ['@uri' => $file->getFileUri()]);
            }
        } catch (\ImagickException $e) {
            \Drupal::logger('image_processor_webp')->error('ImageMagick exception on converting to WebP: @error', ['@error' => $e->getMessage()]);
        } finally {
            $imagick->clear();
            $imagick->destroy();
        }
    }


    private function createImageResource(string $uri, string $type) {
        switch ($type) {
            case 'image/jpeg':
                return imagecreatefromjpeg($uri);
            case 'image/png':
                return imagecreatefrompng($uri);
            default:
                return null;
        }
    }
}
